/*
 * Copyright 2004 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef _SUNWHEEL_H
#define	_SUNWHEEL_H

#pragma ident	"@(#)sunWheel.h	1.1	04/02/03 SMI"

/*
 * Support for scroll wheels on input devices like mice & keyboards
 */

#include <sys/vuid_wheel.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct wheelPrivRec *wheelPrivPtr;

/**
 * Initialize wheels on a VUID interfaced device
 * Uses VUID ioctls on specified file descriptor to query if wheels are 
 * present and to enable wheel events on them if they are.
 * Calls sunWheelInitializeMapsFromOWconfig using specified device name
 * to parse wheelmaps in OWconfig entry for device, using specified defaults
 * for actions not defined in wheelmaps.
 *
 * @param fd			Open file descriptor to VUID device
 * @param device_name		String for XINPUT class entries for this
 *				 device in OWconfig
 * @param default_up		Default action for wheel up events
 * @param default_down		Default action for wheel down events
 * 
 */
wheelPrivPtr sunWheelInitializeVUIDWheels
 (int fd, const char *device_name, int default_up, int default_down);

/**
 * Initialize wheel action maps using specified device name to parse wheelmaps
 * in OWconfig entry for device, using specified defaults for actions not
 * defined in wheelmaps.
 *
 * @param device_name		String for XINPUT class entries for this
 *				 device in OWconfig
 * @param num_wheels		Hint for number of wheels expected
 * @param default_up		Default action for wheel up events
 * @param default_down		Default action for wheel down events
 * 
 */
wheelPrivPtr sunWheelInitializeMapsFromOWconfig
 (const char *device_name, int num_wheels, int default_up, int default_down);

/**
 * Initialize wheel action maps by parsing specified wheelmaps string,
 * using specified defaults for actions not defined in wheelmaps.
 *
 * @param device_name		String representing device for use in errors
 * @param map_string		String containing wheelmap entries to parse
 * @param num_wheels		Hint for number of wheels expected
 * @param default_up		Default action for wheel up events
 * @param default_down		Default action for wheel down events
 * 
 */
wheelPrivPtr sunWheelInitializeMaps
 (const char *device_name, const char *map_string, int num_wheels, 
  int default_up, int default_down);

/**
 * Retrieves the largest mouse button id used in a wheel map 
 *
 * @param pPriv		Wheel mappings for this device
 */
int sunWheelGetMaxMouseButtonId(wheelPrivPtr pPriv);

/**
 * Map VUID wheel action to another VUID event
 *
 * @param pPriv		Wheel mappings for this device
 * @param fe		VUID event to parse
 * @param deltaP	Pointer to int to store delta value in
 *
 * @return int		VUID event to report or -1 to discard
 */
int sunWheelMapVUIDEventToAction
(wheelPrivPtr pPriv, Firm_event *fe, int *deltaP);

/**
 * Map wheel action to another VUID event
 *
 * @param pPriv		Wheel mappings for this device
 * @param wheelId	Id of wheel event is for
 * @param deltaP	In: reported delta, Out: modified delta
 *
 * @return int		VUID event to report or -1 to discard
 */
int sunWheelMapEventToAction
(wheelPrivPtr pPriv, int wheelId, int *deltaP);

/**
 * Free wheel mapping table
 *
 * @param pPriv		Wheel mapping table to free
 */
void sunWheelFreePriv(wheelPrivPtr pPriv);

/**
 * Print debugging messages if XSUNWHEELDEBUG environment variable is set.
 *
 * @param debugLevel	Debugging level to print the message at:
 *			  1 = basic messages
 *			 10 = very verbose
 */
/* PRINTFLIKE2 */
void sunWheelDebugMsg(int debugLevel, const char *msg, ...);

#ifdef __cplusplus
}
#endif

#endif /* _SUNWHEEL_H */
